import { useNuxt, useLogger, addDevServerHandler, createResolver, addTemplate, logger as logger$1, defineNuxtModule, addImportsDir, addImports, addPlugin, addBuildPlugin, hasNuxtModule } from '@nuxt/kit';
import { resolvePackageJSON, readPackageJSON } from 'pkg-types';
import { parseURL, parseQuery, joinURL, hasProtocol, withQuery, withBase } from 'ufo';
import { existsSync } from 'node:fs';
import { pathToFileURL } from 'node:url';
import { createUnplugin } from 'unplugin';
import MagicString from 'magic-string';
import { walk } from 'estree-walker';
import fsp from 'node:fs/promises';
import { lazyEventHandler, eventHandler, createError } from 'h3';
import { fetch } from 'ofetch';
import { colors } from 'consola/utils';
import { defu } from 'defu';
import { join, relative } from 'pathe';
import { hash } from 'ohash';
import { createStorage } from 'unstorage';
import fsDriver from 'unstorage/drivers/fs-lite';
import { isCI, provider } from 'std-env';

const DEVTOOLS_UI_ROUTE = "/__nuxt-scripts";
const DEVTOOLS_UI_LOCAL_PORT = 3030;
function setupDevToolsUI(options, resolve, nuxt = useNuxt()) {
  const clientPath = resolve("./client");
  const isProductionBuild = existsSync(clientPath);
  if (isProductionBuild) {
    nuxt.hook("vite:serverCreated", async (server) => {
      const sirv = await import('sirv').then((r) => r.default || r);
      server.middlewares.use(
        DEVTOOLS_UI_ROUTE,
        sirv(clientPath, { dev: true, single: true })
      );
    });
  } else {
    nuxt.hook("vite:extendConfig", (config) => {
      config.server = config.server || {};
      config.server.proxy = config.server.proxy || {};
      config.server.proxy[DEVTOOLS_UI_ROUTE] = {
        target: `http://localhost:${DEVTOOLS_UI_LOCAL_PORT}${DEVTOOLS_UI_ROUTE}`,
        changeOrigin: true,
        followRedirects: true,
        rewrite: (path) => path.replace(DEVTOOLS_UI_ROUTE, "")
      };
    });
  }
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-scripts",
      // title to display in the tab
      title: "Scripts",
      // any icon from Iconify, or a URL to an image
      icon: "carbon:script",
      // iframe view
      view: {
        type: "iframe",
        src: DEVTOOLS_UI_ROUTE
      }
    });
  });
}

function NuxtScriptAssetBundlerTransformer(options) {
  return createUnplugin(() => {
    return {
      name: "nuxt:scripts:asset-bundler-transformer",
      transformInclude(id) {
        const { pathname, search } = parseURL(decodeURIComponent(pathToFileURL(id).href));
        const { type } = parseQuery(search);
        if (pathname.includes("node_modules/@unhead") || pathname.includes("node_modules/vueuse"))
          return false;
        if (pathname.endsWith(".vue") && (type === "script" || !search))
          return true;
        if (pathname.match(/\.((c|m)?j|t)sx?$/g))
          return true;
        return false;
      },
      async transform(code, id) {
        if (!code.includes("useScript"))
          return;
        const ast = this.parse(code);
        const s = new MagicString(code);
        walk(ast, {
          enter(_node) {
            if (_node.type === "CallExpression" && _node.callee.type === "Identifier" && _node.callee?.name.startsWith("useScript")) {
              const fnName = _node.callee?.name;
              const node = _node;
              let scriptKey;
              let scriptSrcNode;
              let src;
              if (fnName === "useScript") {
                if (node.arguments[0].type === "Literal") {
                  scriptSrcNode = node.arguments[0];
                  scriptKey = scriptSrcNode.value;
                } else if (node.arguments[0].type === "ObjectExpression") {
                  const srcProperty = node.arguments[0].properties.find(
                    (p) => p.key?.name === "src" || p.key?.value === "src"
                  );
                  const keyProperty = node.arguments[0].properties.find(
                    (p) => p.key?.name === "key" || p.key?.value === "key"
                  );
                  scriptKey = keyProperty?.value?.value || srcProperty?.value;
                  scriptSrcNode = srcProperty?.value;
                }
              } else {
                const registryNode = options.registry?.find((i) => i.name === fnName);
                if (!registryNode) {
                  console.warn(`[Nuxt Scripts] Integration ${fnName} not found in registry. Used in ${id}.`);
                  return;
                }
                options.moduleDetected(registryNode.module);
                if (!registryNode.transform && !registryNode.src)
                  return;
                const optionsNode = node.arguments[0];
                if (optionsNode?.type === "ObjectExpression") {
                  const fnArg0 = {};
                  for (const prop of optionsNode.properties) {
                    if (prop.value.type === "Literal")
                      fnArg0[prop.key.name] = prop.value.value;
                  }
                  const srcProperty = node.arguments[0].properties.find(
                    (p) => p.key?.name === "src" || p.key?.value === "src"
                  );
                  if (srcProperty?.value?.value) {
                    scriptSrcNode = srcProperty.value;
                  } else {
                    src = registryNode.src || registryNode.transform?.(fnArg0);
                    if (src === false)
                      return;
                  }
                  scriptKey = registryNode.key;
                }
              }
              if (scriptSrcNode || src) {
                src = src || scriptSrcNode.value;
                if (src) {
                  let canBundle = options.defaultBundle;
                  if (node.arguments[1]?.type === "ObjectExpression") {
                    const assetStrategyProperty = node.arguments[1]?.properties.find(
                      (p) => p.key?.name === "assetStrategy" || p.key?.value === "assetStrategy"
                    );
                    if (assetStrategyProperty) {
                      if (assetStrategyProperty?.value?.value !== "bundle") {
                        canBundle = false;
                        return;
                      }
                      if (node.arguments[1]?.properties.length === 1)
                        s.remove(node.arguments[1].start, node.arguments[1].end);
                      else
                        s.remove(assetStrategyProperty.start, assetStrategyProperty.end);
                      canBundle = true;
                    }
                  }
                  canBundle = canBundle || options.overrides?.[scriptKey]?.assetStrategy === "bundle";
                  if (canBundle) {
                    const newSrc = options.resolveScript(src);
                    if (scriptSrcNode) {
                      s.overwrite(scriptSrcNode.start, scriptSrcNode.end, `'${newSrc}'`);
                    } else {
                      const lastProperty = node.arguments[0].properties[node.arguments[0].properties.length - 1];
                      s.appendRight(lastProperty.end, `, src: '${newSrc}'`);
                    }
                  }
                }
              }
            }
          }
        });
        if (s.hasChanged()) {
          return {
            code: s.toString(),
            map: s.generateMap({ includeContent: true, source: id })
          };
        }
      }
    };
  });
}

const logger = useLogger("@nuxt/scripts");

const ONE_YEAR_IN_SECONDS = 60 * 60 * 24 * 365;
function setupPublicAssetStrategy(options = {}) {
  const assetsBaseURL = options.prefix || "/_scripts";
  const nuxt = useNuxt();
  const renderedScriptSrc = /* @__PURE__ */ new Map();
  const storage = createStorage(fsDriver({
    base: "node_modules/.cache/nuxt/scripts"
  }));
  function normalizeScriptData(src) {
    if (hasProtocol(src, { acceptRelative: true })) {
      src = src.replace(/^\/\//, "https://");
      const url = parseURL(src);
      const file = [
        `${hash(url)}.js`
        // force an extension
      ].filter(Boolean).join("-");
      renderedScriptSrc.set(file, src);
      return joinURL(assetsBaseURL, file);
    }
    return src;
  }
  addDevServerHandler({
    route: assetsBaseURL,
    handler: lazyEventHandler(async () => {
      return eventHandler(async (event) => {
        const filename = event.path.slice(1);
        const url = renderedScriptSrc.get(event.path.slice(1));
        if (!url)
          throw createError({ statusCode: 404 });
        const key = `data:scripts:${filename}`;
        let res = await storage.getItemRaw(key);
        if (!res) {
          res = await fetch(url).then((r) => r.arrayBuffer()).then((r) => Buffer.from(r));
          await storage.setItemRaw(key, res);
        }
        return res;
      });
    })
  });
  if (nuxt.options.dev) {
    nuxt.options.routeRules ||= {};
    nuxt.options.routeRules[joinURL(assetsBaseURL, "**")] = {
      cache: {
        maxAge: ONE_YEAR_IN_SECONDS
      }
    };
  }
  nuxt.options.nitro.publicAssets ||= [];
  const cacheDir = join(nuxt.options.buildDir, "cache", "scripts");
  nuxt.options.nitro.publicAssets.push();
  nuxt.options.nitro = defu(nuxt.options.nitro, {
    publicAssets: [{
      dir: cacheDir,
      maxAge: ONE_YEAR_IN_SECONDS,
      baseURL: assetsBaseURL
    }],
    prerender: {
      ignore: [assetsBaseURL]
    }
  });
  nuxt.hook("nitro:init", async (nitro) => {
    if (nuxt.options.dev)
      return;
    nitro.hooks.hook("rollup:before", async () => {
      await fsp.rm(cacheDir, { recursive: true, force: true });
      await fsp.mkdir(cacheDir, { recursive: true });
      let banner = false;
      for (const [filename, url] of renderedScriptSrc) {
        const key = `data:scripts:${filename}`;
        let res = await storage.getItemRaw(key);
        if (!res) {
          if (!banner) {
            banner = true;
            logger.info("Downloading scripts...");
          }
          let encoding;
          let size = 0;
          res = await fetch(url).then((r) => {
            encoding = r.headers.get("content-encoding");
            size = Number(r.headers.get("content-length") / 1024);
            return r.arrayBuffer();
          }).then((r) => Buffer.from(r));
          logger.log(colors.gray(`  \u251C\u2500 ${url} \u2192 ${joinURL(assetsBaseURL, filename)} (${size.toFixed(2)} kB ${encoding})`));
          await storage.setItemRaw(key, res);
        }
        await fsp.writeFile(join(cacheDir, filename), res);
      }
      if (banner)
        logger.success("Scripts downloaded and cached.");
    });
  });
  return { normalizeScriptData };
}

function extendTypes(module, template) {
  const nuxt = useNuxt();
  const { resolve } = createResolver(import.meta.url);
  addTemplate({
    filename: `modules/${module.replace("/", "-")}.d.ts`,
    getContents: async () => {
      const typesPath = relative(resolve(nuxt.options.rootDir, nuxt.options.buildDir, "module"), resolve("runtime/types"));
      const s = await template({ typesPath });
      return `// Generated by ${module}
${s}
export {}
`;
    }
  });
  nuxt.hooks.hook("prepare:types", ({ references }) => {
    references.push({ path: resolve(nuxt.options.buildDir, `module/${module}.d.ts`) });
  });
}
const isStackblitz = provider === "stackblitz";
async function promptToInstall(name, installCommand, options) {
  if (await resolvePackageJSON(name, { url: options.searchPaths }).catch(() => null))
    return true;
  logger$1.info(`Package ${name} is missing`);
  if (isCI)
    return false;
  if (options.prompt === true || options.prompt !== false && !isStackblitz) {
    const confirm = await logger$1.prompt(`Do you want to install ${name} package?`, {
      type: "confirm",
      name: "confirm",
      initial: true
    });
    if (!confirm)
      return false;
  }
  logger$1.info(`Installing ${name}...`);
  try {
    await installCommand();
    logger$1.success(`Installed ${name}`);
    return true;
  } catch (err) {
    logger$1.error(err);
    return false;
  }
}
const installPrompts = /* @__PURE__ */ new Set();
function installNuxtModule(name, options) {
  if (installPrompts.has(name))
    return;
  installPrompts.add(name);
  const nuxt = useNuxt();
  return promptToInstall(name, async () => {
    const { runCommand } = await import(String("nuxi"));
    await runCommand("module", ["add", name, "--cwd", nuxt.options.rootDir]);
  }, { rootDir: nuxt.options.rootDir, searchPaths: nuxt.options.modulesDir, ...options });
}

const module = defineNuxtModule({
  meta: {
    name: "@nuxt/scripts",
    configKey: "scripts",
    compatibility: {
      nuxt: "^3.11.1",
      bridge: false
    }
  },
  defaults: {
    defaultScriptOptions: {
      assetStrategy: "bundle",
      // Not supported on all scripts, only if the src is static, runtime fallback?
      trigger: "onNuxtReady"
    },
    enabled: true,
    debug: false
  },
  async setup(config, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    const { version, name } = await readPackageJSON(resolve("../package.json"));
    if (!config.enabled) {
      logger.debug("The module is disabled, skipping setup.");
      return;
    }
    nuxt.options.alias["#nuxt-scripts"] = resolve("./runtime/types");
    nuxt.options.runtimeConfig["nuxt-scripts"] = { version };
    nuxt.options.runtimeConfig.public["nuxt-scripts"] = { defaultScriptOptions: config.defaultScriptOptions };
    addImportsDir([
      resolve("./runtime/composables")
    ]);
    nuxt.hooks.hook("modules:done", async () => {
      let registry = [
        {
          name: "useScriptCloudflareWebAnalytics",
          key: "cloudflareWebAnalytics",
          from: resolve("./runtime/registry/cloudflare-web-analytics"),
          src: "https://static.cloudflareinsights.com/beacon.min.js"
        },
        {
          name: "useScriptConfetti",
          key: "confetti",
          from: resolve("./runtime/registry/confetti"),
          src: "https://unpkg.com/js-confetti@latest/dist/js-confetti.browser.js"
        },
        {
          name: "useScriptFacebookPixel",
          key: "facebookPixel",
          from: resolve("./runtime/registry/facebook-pixel"),
          src: "https://connect.facebook.net/en_US/fbevents.js"
        },
        {
          name: "useScriptFathomAnalytics",
          key: "fathomAnalytics",
          from: resolve("./runtime/registry/fathom-analytics"),
          src: false
          // can not be bundled, breaks script
        },
        {
          name: "useScriptHotjar",
          from: resolve("./runtime/registry/hotjar"),
          key: "hotjar",
          transform(options) {
            return withQuery(`https://static.hotjar.com/c/hotjar-${options?.id || ""}.js`, {
              sv: options?.sv || "6"
            });
          }
        },
        {
          name: "useScriptIntercom",
          from: resolve("./runtime/registry/intercom"),
          key: "intercom",
          transform(options) {
            return joinURL(`https://widget.intercom.io/widget`, options?.app_id || "");
          }
        },
        {
          name: "useScriptSegment",
          from: resolve("./runtime/registry/segment"),
          key: "segment",
          transform(options) {
            return joinURL("https://cdn.segment.com/analytics.js/v1", options?.writeKey || "", "analytics.min.js");
          }
        },
        {
          name: "useScriptNpm",
          // key is based on package name
          from: resolve("./runtime/registry/npm"),
          transform(options) {
            return withBase(options?.file || "", `https://unpkg.com/${options?.packageName || ""}@${options?.version || "latest"}`);
          }
        },
        // cloudflare turnstile
        {
          name: "useScriptCloudflareTurnstile",
          key: "cloudflareTurnstile",
          from: resolve("./runtime/registry/cloudflare-turnstile"),
          module: "nuxt-turnstile"
        },
        // third-party-capital
        {
          name: "useScriptGoogleAnalytics",
          key: "googleAnalytics",
          from: resolve("./runtime/registry/google-analytics"),
          module: "@nuxt/third-party-capital"
        },
        {
          name: "useScriptGoogleTagManager",
          key: "googleTagManager",
          from: resolve("./runtime/registry/google-tag-manager"),
          module: "@nuxt/third-party-capital"
        },
        {
          name: "useScriptGoogleMaps",
          key: "googleMaps",
          from: resolve("./runtime/registry/google-tag-manager"),
          module: "@nuxt/third-party-capital"
        }
      ];
      registry = registry.map((i) => {
        i.priority = -1;
        i.module = i.module || "@nuxt/scripts";
        return i;
      });
      addImports(registry);
      await nuxt.hooks.callHook("scripts:registry", registry);
      extendTypes(name, async ({ typesPath }) => {
        return `
declare module '#app' {
  interface NuxtApp {
    ${nuxt.options.dev ? `_scripts: (import('#nuxt-scripts').NuxtAppScript)[]` : ""}
  }
}
declare module '#nuxt-scripts' {
    type NuxtUseScriptOptions = Omit<import('${typesPath}').NuxtUseScriptOptions, 'use' | 'beforeInit'>
    interface ScriptRegistry {
${registry.filter((i) => i.key && i.module !== "@nuxt/scripts").map((i) => {
          const ucFirstKey = i.key.substring(0, 1).toUpperCase() + i.key.substring(1);
          return `        ${i.key}?: import('${i.from}').${ucFirstKey}Input | [import('${i.from}').${ucFirstKey}Input, NuxtUseScriptOptions]`;
        }).join("\n")}
    }
}
`;
      });
      if (config.globals?.length || Object.keys(config.register || {}).length) {
        const template = addTemplate({
          filename: `modules/${name.replace("/", "-")}.mjs`,
          write: true,
          getContents() {
            const imports = ["useScript", "defineNuxtPlugin"];
            const inits = [];
            for (const [k, c] of Object.entries(config.register || {})) {
              const importDefinition = registry.find((i) => i.name === `useScript${k.substring(0, 1).toUpperCase() + k.substring(1)}`);
              if (importDefinition) {
                imports.unshift(importDefinition.name);
                inits.push(`${importDefinition.name}(${JSON.stringify(c)});`);
              }
            }
            return `import { ${imports.join(", ")} } from '#imports'
export default defineNuxtPlugin({
  name: "${name}:init",
  setup() {
${(config.globals || []).map((g) => !Array.isArray(g) ? `    useScript("${g.toString()}")` : g.length === 2 ? `    useScript(${JSON.stringify(g[0])}, ${JSON.stringify(g[1])} })` : `    useScript(${JSON.stringify(g[0])})`).join("\n")}
    ${inits.join("\n    ")}
  }
})`;
          }
        });
        addPlugin({
          src: template.dst
        });
      }
      const scriptMap = /* @__PURE__ */ new Map();
      const { normalizeScriptData } = setupPublicAssetStrategy(config.assets);
      const moduleInstallPromises = /* @__PURE__ */ new Map();
      addBuildPlugin(NuxtScriptAssetBundlerTransformer({
        registry,
        defaultBundle: config.defaultScriptOptions?.assetStrategy === "bundle",
        moduleDetected(module) {
          if (module !== "@nuxt/scripts" && !moduleInstallPromises.has(module) && !hasNuxtModule(module))
            moduleInstallPromises.set(module, () => installNuxtModule(module));
        },
        resolveScript(src) {
          if (scriptMap.has(src))
            return scriptMap.get(src);
          const url = normalizeScriptData(src);
          scriptMap.set(src, url);
          return url;
        }
      }));
      nuxt.hooks.hook("build:done", async () => {
        const initPromise = Array.from(moduleInstallPromises.values());
        for (const p of initPromise)
          await p?.();
      });
    });
    if (nuxt.options.dev)
      setupDevToolsUI(config, resolve);
  }
});

export { module as default };
